#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, Gio
from gi.repository import Unity
import gettext
import locale
import os
from xml.dom import minidom

APP_NAME = 'unity-scope-yelp'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Help.Yelp'
UNIQUE_PATH = '/com/canonical/unity/scope/help/yelp'

SEARCH_HINT = _('Ubuntu Help')
NO_RESULTS_HINT = _('Sorry, there are no Ubuntu Help results that match your search.')
PROVIDER_CREDITS = _('')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-yelp.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'result-help.svg'
DEFAULT_RESULT_MIMETYPE = 'x-scheme-handler/man'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT
HELP_DIR = "/usr/share/help/"

c1 = {'id': 'official',
      'name': _('Official Help Documents'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

EXTRA_METADATA = []


def get_yelp_documents():
    '''
     Parses local help files for <desc> and 1st <title> tags and associates
    them with the page's uri in self.data as: {uri, page description, page title}
    If a help file has no <desc> or <title> tag, it is excluded.
    '''
    mylocale = locale.getlocale()

    data = []
    help_location = '%s%s' % (HELP_DIR, mylocale[0])
    if not os.path.exists(help_location):
        help_location = '%s%s' % (HELP_DIR, mylocale[0][:2])
        if not os.path.exists(help_location):
            help_location = '%s%s' % (HELP_DIR, "C")
    for dirname in os.listdir(help_location):
        directory = help_location + "/" + dirname
        for filename in os.listdir(directory):
            filename = directory + "/" + filename
            if not os.path.isdir(filename):
                if os.path.isfile(filename):
                    xmldoc = minidom.parse(filename)
                    try:
                        pagenode = xmldoc.getElementsByTagName('page')[0]
                        if pagenode.attributes["type"].value == "topic":
                            desc = ""
                            nodes = xmldoc.getElementsByTagName('desc').item(0).childNodes
                            for node in nodes:
                                try:
                                    desc += str(node.wholeText)
                                except:
                                    desc += str(node.childNodes[0].wholeText)
                            desc = desc.strip(' \t\n\r')
                            title = xmldoc.getElementsByTagName('title').item(0).childNodes[0].data
                            if desc == "":
                                desc = title
                            record = []
                            record.append(filename)
                            record.append(desc)
                            record.append(title)
                            record.append(dirname)
                            data.append(record)
                    except:
                        pass
    return data


def search(search, filters):
    '''
    Search for help documents matching the search string
    '''
    results = []
    icon_hint1 = Gio.ThemedIcon.new("help").to_string()
    help_data = get_yelp_documents()
    for data in help_data:
        try:
            if data[3] == "ubuntu-help":
                icon_hint1 = Gio.ThemedIcon.new("distributor-logo").to_string()
            else:
                icon_hint1 = Gio.ThemedIcon.new(data[3]).to_string()
        except:
            pass
        if search == "":
            if not "/ubuntu-help/" in data[0]:
                continue
        if search.lower() in data[1].lower():
            print(data)
            results.append({'uri': data[0],
                            'icon': icon_hint1,
                            'category': 0,
                            'title': data[1],
                            'comment': ""})
        elif search.lower() in data[2].lower():
            print(data)
            results.append({'uri': data[0],
                            'icon': icon_hint1,
                            'category': 0,
                            'title': data[2],
                            'comment': ""})
        elif search.lower() in data[3].lower():
            print(data)
            results.append({'uri': data[0],
                            'icon': icon_hint1,
                            'category': 0,
                            'title': data[1],
                            'comment': ""})
        else:
            continue
    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print(error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
        #if FILTERS:
        #
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se


def load_scope():
    return Scope()
